var targetItem        = 'gms';
var isPlaying         = false;
var animationInterval = 500;
var py_command        = null;
var retrievingItem    = null;
var animationController = null;

function DEBUG( message ) {
    document.getElementById( 'debugDiv' ).innerText = message;
}


// initialization
function init() {
    DEBUG( 'init' );
    var img = document.getElementById( 'graph' );
    if ( img == null ) {
        DEBUG( 'no graph' );
        return;
    }

    if ( window.widget ) {
        if ( widget.preferenceForKey( 'AnimationInterval' ) ) {
            DEBUG( widget.preferenceForKey( 'AnimationInterval' ) );
            animationInterval = parseInt( widget.preferenceForKey( 'AnimationInterval' ) );
        }

        // interval
        var elem = document.getElementById( 'animationSpeed' );
        var children = elem.childNodes;
        for ( var i = 0; i < children.length; i++ ) {
            if ( children[ i ].value && parseInt( children[ i ].value ) == animationInterval ) {
                children[ i ].setAttribute( 'selected' );
                break;
            }
        }
        setTimeout( loadInitialImage, 0 );
    }

    // done button
    createGenericButton( document.getElementById( 'done' ), tr( "Done" ), flipToFront );

}


function setAnimationInterval( interval ) {
    if ( interval > 10 ) {
        animationInterval = interval;
    }
}

function resolveURIs( output ) {
    var URIs = new Array();
    if ( output && output.length > 0 ) {
        var lines = output.split( '\n' );
        var base_uri = lines[ 0 ];
        for ( var i = output.length - 1; i >= 1; i-- ) {
            if ( lines[ i ] && lines[ i ].length > 0 ) {
                var uri = base_uri + lines[ i ] + '.jpg';
                DEBUG( uri );
                URIs.push( uri );
            }
        }
    }
    return URIs;
}

function loadInitialImage() {
    availableImages = "";
    var listCommand = widget.system( "/usr/bin/python getchart.py gms -list", null );
    var output = listCommand.outputString;
    var error = listCommand.errorString;
    if ( output & output.length > 0 ) {
        var lines = output.split( '\n' );
        if ( lines.length > 1 ) {
            var base_uri = lines[ 0 ];
            var uri = base_uri + lines[ 1 ] + '.jpg';
            showGraph( targetItem, uri );
        }
    }
    if ( error && error.length > 0 ) {
        showErrorMessage( error );
    }
    listCommand = null;
    loadGraph( targetItem );
    document.getElementById( 'playButton' ).src = "Images/ButtonPlay1.png";
}

// transit to preference
function flipToBehind(event) {
    var front  = document.getElementById( "front" ); 
    var behind = document.getElementById( "behind" ); 
    widget.prepareForTransition("ToBack"); 
    behind.style.display  = "block"; 
    front.style.display = "none"; 
    
    if (window.widget) {
        setTimeout ('widget.performTransition();', 0);  
    }
}

// transit to main face
function flipToFront(event){
    var front  = document.getElementById( "front" ); 
    var behind = document.getElementById( "behind" ); 
    widget.prepareForTransition("ToFront"); 

    animationInterval = parseInt( document.getElementById( "animationSpeed" ).value );
    behind.style.display  = "none"; 
    front.style.display = "block"; 
    
    if (window.widget) {
        setTimeout ('widget.performTransition();', 0);  
        widget.setPreferenceForKey( animationInterval, 'AnimationInterval' );
    }
    
}

// translation for the future 
function tr( key ) {
    return key;
}

// background loading
function invokeBackgroundLoaders() {
    var int_reload = 60000 * 30;//100000; // 60000 * 30;
    var wait_chart = 300000;//20000; // 120000 ;

    setTimeout( function() { loadChartWithInterval( 'gms', int_reload ); }, wait_chart );
    setTimeout( function() { loadChartWithInterval( 'chart', int_reload ); }, wait_chart * 2 );
    setTimeout( function() { loadChartWithInterval( 'rain', int_reload ); }, wait_chart * 3 );
    setTimeout( function() { loadChartWithInterval( 'temp', int_reload ); }, wait_chart * 4 );

}


function loadChartWithInterval( item, duration ) {
    DEBUG( "set interval " + item  + " " + duration + " in loadChartWithInterval" );
    setInterval( function(){ loadGraph( item ); }, duration );
}

// load graph if no external script are running
function loadGraph( item ) {
    if ( py_command != null ) {
        DEBUG( 'already retrieving ' + retrievingItem + ", then loading '" + item + "' must wait for completion" );
        return;
    }
    
    var command = "/usr/bin/python getchart.py " + item;
    if ( item == targetItem ) {
        document.getElementById( 'graph' ).style.opacity = '0.5';
        document.getElementById( 'messageText' ).innerHTML = "Loading";
    }
    DEBUG( 'Python : ' + command );
    retrievingItem = item.substring( 0, item.length );
    py_command = widget.system( command, newGraphHandler );
}

// cancel current process and load
function forceLoadGraph( item ) {
    if ( py_command != null ) {
        if ( item == retrievingItem ) return;
        py_command.cancel();
    }
    loadGraph( item );
}

// process new graph
function newGraphHandler() {
    DEBUG( 'newGraphHandler' );
    if ( py_command == null ) {
        DEBUG( 'no command' );
        return;
    }
    var output = py_command.outputString;
    var error  = py_command.errorString;
    if ( output ) {
        DEBUG( 'new graph\n' + output );
        var outputs = output.split( "\n" );
        var uri = outputs[ 0 ];
        if ( retrievingItem == targetItem ) {
            showGraph( targetItem, uri );
        } else {
            DEBUG( "store '" + uri + "' to backyard : " + targetItem + '/' + retrievingItem );
        }
    } 
    if ( error && error != undefined && error.length > 0 ) {
        DEBUG( error );
        showErrorMessage( error );
    }
    py_command = null;
    retrievingItem = null;
}


function resolveItemName( item ) {
    if ( item == 'gms' ) {
        return "衛星写真";
    } else if ( item == 'chart' ) {
        return "天気図";
    } else if ( item == 'rain' ) {
        return "降水量";
    } else if ( item == 'temp' ) {
        return "気温";
    } else {
        return "??";
    }
}

function getDateFromURI( uri ) {
    var ps = uri.lastIndexOf( '/' );
    var pp = uri.lastIndexOf( '.' );
    var year = "1900";
    var month = "00";
    var day = "00";
    var hour = "00";
    if ( ps == -1 ) ps = 0;
    if ( pp == -1 ) pp = uri.length;
    if ( pp - ps >= 11 ) {
        year = uri.substring( ps + 1, ps + 5 );
        month = uri.substring( ps + 5, ps + 7 );
        day = uri.substring( ps + 7, ps + 9 );
        hour = uri.substring( ps + 9, ps + 11 );
    }
    return [ year, month, day, hour ];
}

function changeGraph( item ) {
    if ( isPlaying ) {
        DEBUG( 'cannot change while animation' );
        return;
    }
    DEBUG( "chageGraph " + item );
    targetItem = item;
    forceLoadGraph( item );
    return;
}

function showErrorMessage( message ) {
    if ( message == null || message == undefined ) {
        message = "エラー発生";
    }
    document.getElementById( 'graph' ).src = "Images/Empty.png";
    document.getElementById( 'messageText' ).innerText = message;
}

function showGraph( item, uri ) {
    DEBUG( 'showGraph ' + item + " " + uri );
    var tempURI = uri + "?" + (new Date()).getTime();
    var itemName = resolveItemName( item );
    var date = getDateFromURI( uri );
    var graph = document.getElementById( 'graph' );
    
    graph.src = tempURI;
    graph.style.opacity = '1.0';
    DEBUG( 'showgraph ' + tempURI );
    document.getElementById( 'messageText' ).innerHTML = itemName + "<br/>" + date[ 1 ] + "/" + date[ 2 ] + "/" + date[ 3 ];
}




///////////// animation
function playMouseDown( event ) {
    if ( isPlaying ) {
        DEBUG( "STOPPPING" );
        document.getElementById( 'playButton' ).src="Images/ButtonStop2.png";
        window.addEventListener( "mouseup", stopMouseUp, true );
        window.addEventListener( "mouseexit", revertStopButton, true );
        return;
    } else {
        document.getElementById( 'playButton' ).src="Images/ButtonPlay2.png";
        window.addEventListener( "mouseup", playMouseUp, true );
        window.addEventListener( "mouseexit", revertPlayButton, true );
    }
}

function revertPlayButton( event ) {
    document.getElementById( 'playButton' ).src="Images/ButtonPlay1.png";
    window.removeEventListener( "mouseup", playMouseUp, true );
    window.removeEventListener( "mouseexit", revertPlayButton, true );
//     event.stopPropagation(); 
//     event.preventDefault(); 
}

function revertStopButton( event ) {
    document.getElementById( 'playButton' ).src="Images/ButtonStop1.png";
    window.removeEventListener( "mouseup", stopMouseUp, true );
    window.removeEventListener( "mouseexit", revertStopButton, true );
//     event.stopPropagation(); 
//     event.preventDefault(); 
}


function AnimationFrame( item, uris ) {
    this.item  = item;
    this.uris  = uris;
    this.count = 0;
    this.tick  = AnimationFrame__tick;
    this.stop  = AnimationFrame__stop;
    this.timer = null;
}

function AnimationFrame__tick() {
    DEBUG( 'ANIMATION START ' + this.count + " " + this.uris.length + " " + this.uris[ this.count ] );
    showGraph( this.item, this.uris[ this.count ] );
    this.count ++;
    if ( this.count >= this.uris.length ) {
        if ( this.timer ) {
            //DEBUG( 'stop' );
            clearInterval( this.timer );
            this.timer = null;
            isPlaying = false;
            animationController = null;
            document.getElementById( 'playButton' ).src = "Images/ButtonPlay1.png";
        } else {
            DEBUG( "continueing " + this.count );
        }
    }
}

function AnimationFrame__stop() {
    if ( this.timer ) {
        clearInterval( this.timer );
        this.timer = null;
        isPlaying = false;
        animationController = null;
        document.getElementById( 'playButton' ).src="Images/ButtonPlay1.png";
    }
}

// initiate animation
function invokeAnimation() {
    var command = "/usr/bin/python getchart.py " + targetItem + " -list";
    var pycom = widget.system( command, null );
    var output = pycom.outputString;
    var error = pycom.errorString;
    var URIs = resolveURIs( output );

    if ( error ) {
        DEBUG( error );
        showMessage( error );
        return;
    }
    document.getElementById( 'playButton' ).src="Images/ButtonStop1.png";
    animationController = new AnimationFrame( targetItem, URIs );
    isPlaying = true;
    animationController.timer 
        = setInterval( function(){ animationController.tick(); }, animationInterval );
    
}


// stop animation
function stopAnimation() {
    DEBUG( 'stop animation' );
    if ( animationController != null && isPlaying == true ) {
        animationController.stop();
    } else {
        DEBUG( "NOT STOPPED " + isPlaying );
    }
}



//////////// key events

function _mouse_down( event, upHandler, bname, img ) {
    window.addEventListener( 'mouseup', upHandler, true );
    document.getElementById( bname ).src = img;
//     event.stopPropagation(); 
//     event.preventDefault(); 
}

function _mouse_up( event, upHandler, bname, img ) {
    window.removeEventListener( 'mouseup', upHandler, true );
    document.getElementById( bname ).src = img;
//     event.stopPropagation(); 
//     event.preventDefault(); 
}

// receive play button action
function playMouseUp( event ) {
    revertPlayButton( event );
    DEBUG( 'start animation' );
    
    if ( window.widget ) {
        invokeAnimation();
    }
}

// receive stop button action
function stopMouseUp( event ) {
    DEBUG( "STOPPING ....." );
    revertStopButton( event );
    stopAnimation();
}

function gmsMouseDown( event ) {
    if ( isPlaying ) return;
    _mouse_down( event, gmsMouseUp, "gmsButton", "Images/ButtonGMS2.png" );
}

function gmsMouseUp( event ) {
    _mouse_up( event, gmsMouseUp, "gmsButton", "Images/ButtonGMS1.png" );
    changeGraph( 'gms' );
}


function chartMouseDown( event ) {
    if ( isPlaying ) return;
    _mouse_down( event, chartMouseUp, "chartButton", "Images/ButtonChart2.png" );
}

function chartMouseUp( event ) {
    _mouse_up( event, chartMouseUp, "chartButton", "Images/ButtonChart1.png" );
    changeGraph( 'chart' );
}

function rainMouseDown( event ) {
    if ( isPlaying ) return;
    _mouse_down( event, rainMouseUp, "rainButton", "Images/ButtonRain2.png" );
}

function rainMouseUp( event ) {
    _mouse_up( event, rainMouseUp, "rainButton", "Images/ButtonRain1.png" );
    changeGraph( 'rain' );
}

function tempMouseDown( event ) {
    if ( isPlaying ) return;
    _mouse_down( event, tempMouseUp, "tempButton", "Images/ButtonTemp2.png" );
}

function tempMouseUp( event ) {
    _mouse_up( event, tempMouseUp, "tempButton", "Images/ButtonTemp1.png" );
    changeGraph( 'temp' );
}
